%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% The duality of the Italian economy 
% MATLAB routines to (1) compute the wedges, (2) construct the graphs and 
% (3) perform counterfactuals 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%% Load empirical data 

load(['masterdata.mat'])

%% Pre-set some of the parameter values 
PAR.alpha       = 1.5;            % utility of leisure parameter
PAR.theta       = theta_avg;      % capital share of production in the official sector
PAR.thetatilde  = theta_avg;      % capital share of production in the unofficial sector
PAR.delta       = delta;          % depreciation rate computed from perpetual inventory
PAR.beta        = 0.96;           % average annual time discount rate
PAR.taux_slope  = 1.0;            % elasticity unofficial productivity wedge 
PAR.taub_slope  = 1.0;            % elasticity cost foreign investment 

%% Define region and sector indices
area = regions(1:2);
sec = cell({'trade', 'notrade'});

%% Compute shares tradable and nontradables and rescale prices
FinalPrice_Correction=1.0;
for aa = 1:length(area)
  for ss = 1:length(sec)
    sectoral_mean.(area{1,aa}).(sec{1,ss})=mean(nva.(area{1,aa}).(sec{1,ss}),'omitnan');
  end
  share_NT.(area{1,aa})=sectoral_mean.(area{1,aa}).notrade/(sectoral_mean.(area{1,aa}).notrade+sectoral_mean.(area{1,aa}).trade)  ;
end
NTprice_Correction=(FinalPrice_Correction+share_NT.sou-1)/share_NT.sou;
p.sou=p.sou*FinalPrice_Correction;
pt.sou.notrade=pt.sou.notrade*NTprice_Correction;


%% Rename variables with model notation
for aa = 1:length(area)
  for ss = 1:length(sec)
    L.(area{1,aa}).(sec{1,ss}) = h.(area{1,aa}).(sec{1,ss}) .* ...
                   ( empreg.(area{1,aa}).(sec{1,ss}) ./  emptot.(area{1,aa}).(sec{1,ss}) );               %Total hours in regular sectors 
    Ltilde.(area{1,aa}).(sec{1,ss}) = h.(area{1,aa}).(sec{1,ss}) .* ...
                   ( irreg.(area{1,aa}).(sec{1,ss}) ./  emptot.(area{1,aa}).(sec{1,ss}) );                %Total hours in irregular sectors
    Kbar.(area{1,aa}).(sec{1,ss}) = rk.(area{1,aa}).(sec{1,ss});                                          %Total real capital
    X.(area{1,aa}).(sec{1,ss}) = nva.(area{1,aa}).(sec{1,ss}) ./ pt.(area{1,aa}).(sec{1,ss});             %Regular real value added
    Xtilde.(area{1,aa}).(sec{1,ss}) = nva_under.(area{1,aa}).(sec{1,ss}) ./ pt.(area{1,aa}).(sec{1,ss});  %Irregular real value added
    PT.(area{1,aa}).(sec{1,ss}) = pt.(area{1,aa}).(sec{1,ss}) ./ p.(area{1,aa});                          %Price in units of local final goods
  end    
  C.(area{1,aa}) = nct.(area{1,aa}) ./ p.(area{1,aa});                                                    % Real consumption
  G.(area{1,aa}) = ngt.(area{1,aa}) ./ p.(area{1,aa});                                                    % Real government purchases
  TB.(area{1,aa}) = ntbt.(area{1,aa}) ./ p.(area{1,aa});                                                  % Real trade balance
end

%% Relative population
pop.tot=pop.nor+pop.sou;
pop_share_nor=pop.nor./pop.tot;
PAR.mu(1) = mean(pop_share_nor,'omitnan');
PAR.mu(2) = 1-PAR.mu(1);

%% Redefine variables in per-capita terms
for aa = 1:length(area)
  for ss = 1:length(sec)
    Kbar.(area{1,aa}).(sec{1,ss})   = Kbar.(area{1,aa}).(sec{1,ss}) ./ pop.(area{1,aa}); 
    X.(area{1,aa}).(sec{1,ss})      = X.(area{1,aa}).(sec{1,ss}) ./ pop.(area{1,aa}); 
    Xtilde.(area{1,aa}).(sec{1,ss}) = Xtilde.(area{1,aa}).(sec{1,ss}) ./ pop.(area{1,aa}); 
  end
  C.(area{1,aa}) = C.(area{1,aa}) ./ pop.(area{1,aa}); 
  G.(area{1,aa}) = G.(area{1,aa}) ./ pop.(area{1,aa}); 
  TB.(area{1,aa}) = TB.(area{1,aa}) ./ pop.(area{1,aa}); 
end

%% Express worked hours as a fraction of total hours available in a year for a working age individual 
avail_hours_per_worker = 16*312;
for aa = 1:length(area)
  for ss = 1:length(sec)
    L.(area{1,aa}).(sec{1,ss})       = L.(area{1,aa}).(sec{1,ss}) ./ (avail_hours_per_worker*pop.(area{1,aa})) ;     %Per-capita share of regular time worked in each region/sector
    Ltilde.(area{1,aa}).(sec{1,ss})  = Ltilde.(area{1,aa}).(sec{1,ss}) ./ (avail_hours_per_worker*pop.(area{1,aa})); %Per-capita share of irregular time worked in each region/sector
  end
end

%% Split capital stock in official and unofficial

% Proxy for wedge ratios in regular-irregular segments of the economy
for aa = 1:length(area)
  for ss = 1:length(sec)
    proxy_wedge_taux.(area{1,aa}).(sec{1,ss}) = (Xtilde.(area{1,aa}).(sec{1,ss}) ./ Ltilde.(area{1,aa}).(sec{1,ss})) ./ ...
                            (X.(area{1,aa}).(sec{1,ss}) ./ L.(area{1,aa}).(sec{1,ss}));
  end
end

% Capital allocation
for aa = 1:length(area)
  for ss = 1:length(sec)
    X_ratio=Xtilde.(area{1,aa}).(sec{1,ss})./ X.(area{1,aa}).(sec{1,ss});
    L_ratio=Ltilde.(area{1,aa}).(sec{1,ss})./ L.(area{1,aa}).(sec{1,ss});
    K.(area{1,aa}).(sec{1,ss}) = Kbar.(area{1,aa}).(sec{1,ss})./ ...
                        ( 1 + (X_ratio./(proxy_wedge_taux.(area{1,aa}).(sec{1,ss}).*L_ratio.^(1-PAR.theta))).^(1/PAR.theta) ) ;                       
    Ktilde.(area{1,aa}).(sec{1,ss})=Kbar.(area{1,aa}).(sec{1,ss})-K.(area{1,aa}).(sec{1,ss});
  end
end

%% Set share parameter in the production of final goods

% Set production inputs of tradable and nontradable 
for aa = 1:length(area)
  for ss = 1:length(sec)
    M.(area{1,aa}).(sec{1,ss})=X.(area{1,aa}).(sec{1,ss})+Xtilde.(area{1,aa}).(sec{1,ss});  
  end
  Y.(area{1,aa})=M.(area{1,aa}).trade.*PT.(area{1,aa}).trade + M.(area{1,aa}).notrade.*PT.(area{1,aa}).notrade;  
  share_M_trade.(area{1,aa}) = M.(area{1,aa}).trade .* PT.(area{1,aa}).trade ./ Y.(area{1,aa});
  PAR.eta(aa)=mean(share_M_trade.(area{1,aa}),'omitnan');
end  

% Use average share for both regions
PAR.eta(1)=(mean(share_M_trade.nor,'omitnan')+mean(share_M_trade.sou,'omitnan'))/2;
PAR.eta(2)=(mean(share_M_trade.nor,'omitnan')+mean(share_M_trade.sou,'omitnan'))/2;

%%
%=========================================================================
%=========================================================================
% COMPUTATION OF WEDGES
%=========================================================================
%=========================================================================

%=========================================================================
% Wedge for labor
%=========================================================================

for aa = 1:length(area)
  for ss = 1:length(sec)   
    wedge_l_off.(area{1,aa}).(sec{1,ss}) = ((1-PAR.theta)/PAR.alpha) .* ...
            ( ( 1 - L.(area{1,aa}).trade - L.(area{1,aa}).notrade-Ltilde.(area{1,aa}).trade - Ltilde.(area{1,aa}).notrade ) ./ L.(area{1,aa}).(sec{1,ss}) ) .*...
            ( X.(area{1,aa}).(sec{1,ss}) .* PT.(area{1,aa}).(sec{1,ss}) ./ (C.(area{1,aa}) + G.(area{1,aa})) ) - 1;  
    wedge_l_unoff.(area{1,aa}).(sec{1,ss}) = ((1-PAR.theta)/PAR.alpha) .* ...
            ( ( 1 - L.(area{1,aa}).trade - L.(area{1,aa}).notrade-Ltilde.(area{1,aa}).trade - Ltilde.(area{1,aa}).notrade ) ./ Ltilde.(area{1,aa}).(sec{1,ss}) ) .*...
            ( Xtilde.(area{1,aa}).(sec{1,ss}) .* PT.(area{1,aa}).(sec{1,ss}) ./ (C.(area{1,aa}) + G.(area{1,aa})) ) - 1;  
    Ave_wedge_l_off(aa,ss) = mean(wedge_l_off.(area{1,aa}).(sec{1,ss}),'omitnan'); 
    Ave_wedge_l_unoff(aa,ss) = mean(wedge_l_unoff.(area{1,aa}).(sec{1,ss}),'omitnan'); 
  end
end

%=========================================================================
% Wedge for capital
%=========================================================================

for aa = 1:length(area)
  for ss = 1:length(sec)
    wedge_k_off.(area{1,aa}).(sec{1,ss}) = ...
        PAR.theta * (lagmatrix(X.(area{1,aa}).(sec{1,ss}) .* PT.(area{1,aa}).(sec{1,ss}),-1) ./ lagmatrix(K.(area{1,aa}).(sec{1,ss}),-1)) ./ ...
       (lagmatrix((C.(area{1,aa}) + G.(area{1,aa})),-1) ./ (PAR.beta*(C.(area{1,aa}) + G.(area{1,aa}))) + PAR.delta -1 ) - 1;
    wedge_k_unoff.(area{1,aa}).(sec{1,ss}) = ...
        PAR.thetatilde * (lagmatrix(Xtilde.(area{1,aa}).(sec{1,ss}) .* PT.(area{1,aa}).(sec{1,ss}) ,-1) ./ lagmatrix(Ktilde.(area{1,aa}).(sec{1,ss}),-1)) ./ ...
       (lagmatrix((C.(area{1,aa}) + G.(area{1,aa})),-1) ./ (PAR.beta*(C.(area{1,aa}) + G.(area{1,aa})) ) + PAR.delta -1 ) - 1;
    Ave_wedge_k_off(aa,ss) = mean(wedge_k_off.(area{1,aa}).(sec{1,ss}),'omitnan'); 
    Ave_wedge_k_unoff(aa,ss) = mean(wedge_k_unoff.(area{1,aa}).(sec{1,ss}),'omitnan'); 
  end
end

%=========================================================================
% Productivity intermediate production and wedge
%=========================================================================

for aa = 1:length(area)
  for ss = 1:length(sec)
    IntProd_off.(area{1,aa}).(sec{1,ss}) = X.(area{1,aa}).(sec{1,ss}) ./ ...
                          (K.(area{1,aa}).(sec{1,ss}).^PAR.theta .*  L.(area{1,aa}).(sec{1,ss}).^(1-PAR.theta)) ;     
    IntProd_unoff.(area{1,aa}).(sec{1,ss}) = Xtilde.(area{1,aa}).(sec{1,ss}) ./ ...
                          (Ktilde.(area{1,aa}).(sec{1,ss}).^PAR.theta .* Ltilde.(area{1,aa}).(sec{1,ss}).^(1-PAR.theta)) ; 
    wedge_x.(area{1,aa}).(sec{1,ss}) = 1- IntProd_unoff.(area{1,aa}).(sec{1,ss})./IntProd_off.(area{1,aa}).(sec{1,ss});
    Ave_IntProd(aa,ss) = mean(IntProd_off.(area{1,aa}).(sec{1,ss}),'omitnan'); 
    Ave_wedge_x(aa,ss) = mean(wedge_x.(area{1,aa}).(sec{1,ss}),'omitnan'); 
  end
end

%=========================================================================
% Productivity final production
%=========================================================================

for aa = 1:length(area)
  FinProd.(area{1,aa}) = Y.(area{1,aa}) ./ (M.(area{1,aa}).trade.^PAR.eta(aa) .* M.(area{1,aa}).notrade.^(1-PAR.eta(aa)));
  Ave_FinProd(aa) = mean(FinProd.(area{1,aa}),'omitnan');   
end

%=========================================================================
% Bond wedge
%=========================================================================

for aa = 1:length(area)
  wedge_b.(area{1,aa}) = 1 - ( ( PAR.beta^(-1) * lagmatrix((C.(area{1,aa}) + G.(area{1,aa})),-1)./(C.(area{1,aa}) + G.(area{1,aa})) ) .* ...
                      ((1./(1+int10_ita.real/100)) .* PT.(area{1,aa}).trade./ lagmatrix(PT.(area{1,aa}).trade,-1))   );
  Ave_wedge_b(aa) = mean(wedge_b.(area{1,aa}),'omitnan'); 
end

%=========================================================================
% Residual in budget constraint and net transfers
%=========================================================================

for aa = 1:length(area)
  ResBC.(area{1,aa}) = Y.(area{1,aa}) - C.(area{1,aa}) ...
                     - PAR.delta .* (K.(area{1,aa}).trade + Ktilde.(area{1,aa}).trade + K.(area{1,aa}).notrade + Ktilde.(area{1,aa}).notrade );
  Ave_GovPur(aa) = mean(G.(area{1,aa}),'omitnan');
  Ave_GovTra(aa) = mean(ResBC.(area{1,aa}),'omitnan') - Ave_GovPur(aa);
end

Ave_GovTra(1) = - Ave_GovTra(2)*PAR.mu(2)/PAR.mu(1);

%%
%=========================================================================
%=========================================================================
% Print net transfers paid to other region as ratio of output
%=========================================================================
%=========================================================================

%a1=['North region transfer to South= ', num2str(Ave_GovTra(1)/mean(Y.(area{1,1}),'omitnan'))];
%a2=['South region transfer to North= ', num2str(Ave_GovTra(2)/mean(Y.(area{1,2}),'omitnan'))];
%disp(a1);
%disp(a2);


%%
%===============================================================================
%===============================================================================
% FUNCTIONS
%===============================================================================
%===============================================================================

function y = ProdFun(m_T,m_NT,FinProd,eta)
  y = FinProd*abs(m_T)^eta*abs(m_NT)^(1-eta); 
end

function mpr1 = ProdFun_Der1(m_T,m_NT,FinProd,eta)
  mpr1 = eta*FinProd*abs(m_T)^(eta-1)*abs(m_NT)^(1-eta); 
end

function mpr2 = ProdFun_Der2(m_T,m_NT,FinProd,eta)
  mpr2 = (1-eta)*FinProd*abs(m_T)^eta*abs(m_NT)^(-eta); 
end

function taux = tauxFun(taux_const,taux_slope,X,Xtilde)
  taux = taux_const + taux_slope*(Xtilde/(X+Xtilde)); 
end

function taub = taubFun(taub_const,taub_slope,mmu,BB)
  taub = taub_const + taub_slope*mmu*BB; 
end


